#include <system.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "button_lib.h"
#include "LCD_lib.h"
#include "A2DConv.h"
#include "generator_commands.h"
#include "GPS_parse_lib.h"

//A_D Conversion Bits//
volatile bit ad_convert@ADCON0.1;       //GO_DONE status bit
volatile bit ad_enable@ADCON0.0;		//ADC Enable bit
volatile bit csb0@ADCON0.2;             //Channel Select Bit 0
volatile bit csb1@ADCON0.3;             //Channel Select Bit 1
volatile bit csb2@ADCON0.4;             //Channel Select Bit 2
volatile bit csb3@ADCON0.5;             //Channel Select Bit 3
//A_D Conversion Signals//
unsigned short current_1;		//Port A0
unsigned char uc_voltage; 		//Port A1
unsigned char uc_voltage2;
unsigned short current_2;		//Port A2
unsigned char gen_out;			//Port A3

//Initialize A2D Conversions//
void A2D_init(void)
{
	//adif = 0;
	//adie = 1;
	trisa.0 = 1;
	trisa.1 = 1;
	trisa.2 = 1;
	trisa.3 = 1;
	//ADCON0 Register Bits
		//Bit 7-6 Unimplemented
		//Bit 5-2 Voltaile Channel Select Bits
		//Bit 1 Volatile Status
		//Bit 0 ADC Enable Bit
			adcon0= 00000000b;	
	//ADCON1 Register Bits
		//Bit 7-6 Unimplemented
		//Bit 5-4 Voltage Reference(Chosen as Vref+=Vdd and Vref-=Vss)
		//Bit 3-0 Port Configuration Bits (1011 = AN0-AN3 Analog Inputs)
			adcon1= 00001011b;
	//ADCON2 Register Bits
		//Bit 7 A/D Result Formal Select Bit (0=left justified)
		//Bit 6 Unimplemented
		//Bit 5-3 Acquisition Time Select Bit (111= 20TAD)
		//Bit 2-0 ADC Clock Select (101= Fosc/16)
			adcon2= 00111101b;
	current_1 = 0;		
	uc_voltage = 0;
	uc_voltage2 = 0;
	current_2 = 0;		
	gen_out = 0;	
	return;
}
//ADConversion for Current into Ultracaps
unsigned short current_in(void)
{
	//Channel Select Bits ADCON0 5:2 for Pin A0//
	csb3 = 0; csb2 = 0; csb1 = 0; csb0 = 0;   

	ad_enable = 1;
	ad_convert = 1; 
	delay_us(10);
	while (ad_convert);//Wait For Conversion to Finish
	ad_enable = 0;
	unsigned short temp = 0xffff;					//Need to be more specific for current A/D so use all 10 bits
	temp &= adresh;									//Store MSB first and shift right 8 bits.
	current_1 = ((temp << 8) | (adresl&0x00ff));	//Store LSB and create short that is shifted left 6 bits.
	current_1 = ((current_1 >> 6) & 0x03ff);		//Shift back 6 bits to right so 10 bit A/D conversion can be passed.
	return current_1;

//ADConversion for Ultracap Voltage//
//Used to sample varying rectified signal when Ultracaps are charging//
unsigned char ultracap_voltage(void)
{
	csb3 = 0; csb2 = 0; csb1 = 0; csb0 = 1;   
	short sample =0;
	char volts;
	for (int multi = 0; multi<10; multi++)
	{
		ad_enable = 1;
		delay_us(10); 
		ad_convert = 1;
		while (ad_convert);//Wait For Conversion to Finish
		ad_enable = 0;
		volts = adresh;
		sample = sample + volts;
		adresh=0;
	}
	sample = sample/10;
	uc_voltage = sample;
	return uc_voltage;
} 
//Used to not confuse main and interrupt calling same function.
unsigned char ultracap_voltage2(void)
{
	csb3 = 0; csb2 = 0; csb1 = 0; csb0 = 1;   
	short sample2 =0;
	for (int multi = 0; multi<10; multi++)
	{
		ad_enable = 1;
		ad_convert = 1;
		delay_us(10); 
		while (ad_convert);//Wait For Conversion to Finish
		ad_enable = 0;
		uc_voltage4 = adresh;
		sample2 += uc_voltage4;
		adresh=0;
	}
	sample2 = sample2/10;
	uc_voltage2 = sample2;
	return uc_voltage2;
} 
//ADConversion for Current out of Ultracaps//
unsigned short current_out(void)
{
	//Channel Select Bits ADCON0 5:2 for Pin A2//
	csb3 = 0; csb2 = 0; csb1 = 1; csb0 = 0;   

	ad_enable = 1;
	ad_convert = 1; 
	delay_us(10);
	while (ad_convert);//Wait For Conversion to Finish
	ad_enable = 0;
	unsigned short temp = 0xffff;
	temp &= adresh;								//
	current_2 = ((temp << 8) | (adresl&0x00ff));
	current_2 = ((current_2 >> 6) & 0x03ff);
	return current_2;
}
//ADConversion for Generator Signal//
unsigned char gen_output(void)
{
	//Channel Select Bits ADCON0 5:2 for Pin A3//
	csb3 = 0; csb2 = 0; csb1 = 1; csb0 = 1;   

	ad_enable = 1;
	ad_convert = 1; 
	delay_us(10);
	while (ad_convert);//Wait For Conversion to Finish
	ad_enable = 0;
	gen_out = adresh;
	adresh=0;
	return gen_out;
}