#include <system.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "button_lib.h"
#include "LCD_lib.h"
#include "A2DConv.h"
#include "generator_commands.h"
#include "GPS_parse_lib.h"

//GPS Receive Interrupt Signals//
volatile bit rcif@PIR1.5;
volatile bit rcie@PIE1.5;
volatile bit rcip@IPR1.5;
//GPS Send/Receive Signals
volatile bit txen@TXSTA.5;
volatile bit trmt@TXSTA.1;
volatile bit cren@RCSTA.4;

char gpsout;
char gpsdataR[100];
char gpsdataG[100];
unsigned char num;
char dat;
int tempR;
int tempG;
int getstr;
volatile int RMC;
volatile int GGA;
int commaR;
int commaG;
int commaRpl[12];
int commaGpl[14];
int start;
int stop;
char speed[11];
int speedcount;
int decimal;
char lat[21];
int latcount;
char lon[22];
int loncount;
char time[15];
int timecount;
int endspeedcount;
char speedtopass[4];
int speedknots;
int speedmph;
bool oktoparse;


void init_uart(void)
{	
	//Set Baud rate//Initialize PortC USART
	spbrg = 00010000b;
	spbrgh = 00000100b;
	
	txsta = 00100110b;
	baudcon = 01001000b;//16-bit generator
	rcsta = 10000000b;
	return;
}
void putc(char value) //Send an ASCII character to Port C
{
	volatile bit txen@TXSTA.5;
	volatile bit trmt@TXSTA.1;
	txen = 1;
	while(true)
	{
		if(trmt)
		{
			txreg = value;
			return;
		}
	}
}
void GPS_init_strings(void)
{
	//Initialize GPS by sending it this string//
	char string1[] = "$PSRF100,1,4800,8,1,0*0E\r\n";
	for (int i=0; i<strlen(string1); i++)
	{
		char x = string1[i];
		putc(x);
	}
	//Turn off unwanted GSV and GSA Messages//
	char string3[] = "$PSRF103,2,0,0,1*26\r\n";
	for (int l=0; l<strlen(string3); l++)
	{
		char m = string3[l];
		putc(m);
	}
	char string4[] = "$PSRF103,3,0,0,1*27\r\n";
	for (int n=0; n<strlen(string4); n++)
	{
		char o = string4[n];
		putc(o);
	}

}
void GPS_init()
{
	init_uart();
	delay_s(1);
	GPS_init_strings();
	GPS_init_strings();
	GPS_init_strings();
	GPS_init_strings();
	GPS_init_strings();//Send these strings multiple times to ensure that GPS receives them.
	//GPS Variables//
		num = 0;
		rcie = 0;
		rcip = 1;
		getstr = 0;
	//Parsing Variables//
		RMC = 0;
		tempR = 10;
		GGA = 0;
		commaR = 0;
		commaG = 0;
		start= 0;
		stop =0;
	//Get Speed Variables//
		speed[0] = 'S';speed[1] = 'p';speed[2] = 'e';speed[3] = 'e';speed[4] = 'd';
		speed[5] = ' ';speed[6] = '=';
		speedcount = 7;
		decimal = 0;
		speedknots = 0;
	//Get Latitude Variables//
		lat[0]='L';lat[1]='a';lat[2]='t';lat[3]='i';lat[4]='t';
		lat[5]='u';lat[6]='d';lat[7]='e';lat[8]=':';
		latcount = 9;
	//Get Longitude Variables//
		lon[0]='L';lon[1]='o';lon[2]='n';lon[3]='g';lon[4]='i';
		lon[5]='t';lon[6]='u';lon[7]='d';lon[8]='e';lon[9]=':';
		loncount = 10;
	//Get Time Variables//
		time[0]='T';time[1]='i';time[2]='m';time[3]='e';time[4]=' ';
		timecount = 5;
	cren = 0;
	oktoparse = 0;
	return;
}
//Get time by parsing between 0 and 1st commas//
void gettimeR(void)	//store gps output between comma 0-1 in time[] string
{	
	start = commaRpl[0]; start++;
	stop = commaRpl[1];
	for (start; start<stop; start++)
	{	
		time[timecount] = gpsdataR[start]; 		
		timecount++;
	}
	return;
}
//Get latitude by parsing between 2nd and 3rd (N/S between 3rd and 4th)//
void getLatR(void)
{
	start = commaRpl[2]; start++;
	stop = commaRpl[4];
	for (start; start<stop; start++)
	{	
		lat[latcount] = gpsdataR[start]; 		
		latcount++;
	}
	for (latcount; latcount <21; latcount++)
		lat[latcount] = ' ';
	return;
}
//Get longitude by parsing gps between 4th and 5th commas (E/W between 5th and 6th)
void getLonR(void)
{
	start = commaRpl[4]; start++;
	stop = commaRpl[6];
	for (start; start<stop; start++)
	{	
		lon[loncount] = gpsdataR[start]; 		
		loncount++;
	}
	for (loncount; loncount <22; loncount++)
		lon[loncount] = ' ';
	return;
}
//Read the speed from the gps by parsing between 6th and 7th commas//
void getspeed(void)
{
	start = commaRpl[6]; start++;
	stop = commaRpl[7];
	int decimalcount = 0;
	int speedpass = 0;
	decimal = decimalcount;
	for(start; start<stop; start++)
	{
		if (gpsdataR[start] == '.')
			decimal = decimalcount;		//to be used to scale speed
		decimalcount++;
		if (gpsdataR[start] != '.')	//store the numbers in the speed value
		{
			speed[speedcount] = gpsdataR[start];
			speedtopass[speedpass] = gpsdataR[start];
			speedpass++;
			speedcount++;
		}
	}
	for(speedcount; speedcount<11; speedcount++)
		speed[speedcount] = ' ';
	speedknots = atoi(speedtopass);
	return;
}
int speedtopass(void) //return speed in mph by scaling speed in knots without decimal place
{
	speedmph = speedknots*115;
	speedmph = speedmph/100;
	return speedmph;
}
int decimal_to_disp(void)
{
	return decimal;
}
void readGPS(char read)
{
	gpsout = rcreg;
	if (gpsout == '$') 
	{
		getstr = 1;
		commaR = 0;
		commaG = 0;
		num = 0;
		while (getstr==1)
		{	
			num++;
			cren = 1;
			while(!rcif);
			gpsout = rcreg;
			if (num==3)
			{
				if (gpsout =='G')
				{
					GGA=0;
					return;
				}
				if (gpsout =='R')
					RMC=1;
				getstr=0;
			}
		}
	}
	while(RMC==1)
	{
		
		if (gpsout == ',')
		{
			commaRpl[commaR] = num;
			commaR++;
		}
		if (num==99 || gpsout == '*')
		{
			gpsdataR[num] = ' ';
			RMC = 0;
			num = 0;
			getspeed();
			getLatR();
			getLonR();
		}
		if (gpsout != '*')
		{
			gpsdataR[num] = gpsout;
			num++;
		}
		cren=1;
		while(!rcif);
		gpsout=rcreg;
	}
	return;
}
//Function for printing longitude//
void LCD_printLon(void)
{
	for (int printLon = 0; printLon<21; printLon++)
	{
		if(printLon == 10) LCD_cursor(0x43);
		LCD_putchar(lon[printLon]);
	}
	return;
}
//Function for printing latitude//
void LCD_printLat(void)
{
	for (int printLat = 0; printLat<21; printLat++)
	{
		if(printLat == 9) LCD_cursor(0x43);
		LCD_putchar(lat[printLat]);
	}
	return;
}
