#include <system.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "button_lib.h"
#include "LCD_lib.h"
#include "A2DConv.h"
#include "generator_commands.h"
#include "GPS_parse_lib.h"


volatile bit LCD_cs@LATC.1;				//low true chip selects


//initialize spi for lcd
void LCD_init(void)
{
	//Setting Timer 2//
		//Bit7 = 0 : Unimplemented
		//Bit6:3 = 0000 : Postscale Bits, 1:1 Postscale
		//Bit2 = 1 : TMR2 on
		//Bit1:0 = 11 : Prescalar 1:1 ratio
	t2con = 00000100b;
	
	//Initially Clear all//
	sspcon1 = 00000000b;
	
	//Set Bits for I/O Port C
		//Bit7 = 1 : EUSART Recieve Input
		//Bit6 = 0 : EUSART Transmit Output
		//Bit5 = 0 : SPI Data Output cleared (MOSI)
		//Bit4 = 1 : SPI Data Input Auto-Controlled by SSPEN (MISO)
		//Bit3 = 0 : SCK (Master Mode=0) (Slave Mode=1)
		//Bit2 = 0 : Irrelevant
		//Bit1 = 0 : Chip Select Bit for SD Card
		//Bit0 = 0 : Chip Select Bit for LCD
	trisc = 10010000b;
	
	//Set Bits for SSP1STAT Register
		//Bit7 = 0 : SPI Master Sample Time in Middle of output time
		//Bit6 = 0 : clock select bit:transmit on idle to active transition
		//Bit5:1 = 0 : Used for I2C mode							
		//Bit0 = 0 : SSP1BUF transmit complete bit when = 1
	sspstat = 00000000b;
		
	//Set Bits For SSP1CON1 Register
		//Bit7 = 0 : Write Collision Detect
		//Bit6 = 0 : Receive Overflow Indicator
		//Bit5 = 1 : SPI enable Serial ports
		//Bit4 = 1 : Clock Parity Select (1=idle-high;0=idle-low)
		//Bit3:0 = 0011 : (Master Mode, TMR2 output/2)
	sspcon1 = 00110011b;

	LCD_cs = 1;  	//Chip select high false
	LCD_clear();
	return;
}


//Print Character to SPI LCD Screen by Sending 1 Byte through SPI//
char LCD_putchar(char data)
{
	LCD_cs = 0;  			//Chip select true??
	sspbuf = data;			//BUF register sends out data
	while (!sspstat.BF);	//Wait for IF to say transmission/receive complete
	LCD_cs = 1;				//Chip select high/false??
	delay_ms(1);
	return sspbuf;
}

//Print a character to the screen//
void LCD_printf(const char* text )
{
	char i = 0;
	while( *text != 0 )
	{
	LCD_putchar( *text++ );
	}
	return;
}
//Print the Speed from the GPS parsing//
void LCD_speed(void)
{
	int data = speedtopass();
	int decimal = decimal_to_disp();
	int tens;
	int ones;
	int tenth;
	int hundth;
	tens  = data/1000;	//Tens Place
	data  = data%1000;	//Remainder for Tens Place
	ones  = data/100;	//Ones Place
	data  = data%100;
	tenth = data/10;	//Tenths Place
	hundth= data%10;	//Hundreths Place
	
	switch (decimal)//to know where to place the decimal//
	{
		case 0:
			LCD_putchar(ones + '0');
			LCD_putchar(0x2E);
			LCD_putchar(tenth + '0');
			LCD_putchar(hundth + '0');
			break;
		case 1:
			LCD_putchar(ones + '0');
			LCD_putchar(0x2E);
			LCD_putchar(tenth + '0');
			LCD_putchar(hundth + '0');
			break;
		case 2:
			LCD_putchar(tens +'0');
			LCD_putchar(ones + '0');
			LCD_putchar(0x2E);
			LCD_putchar(tenth + '0');
			LCD_putchar(hundth + '0');
			break;		
	}
	return;
}

//Print unsigned long with decimal in correct place//
void LCD_voltage(unsigned char voltage)
{
	unsigned long data;
	data = voltage*15400;
	data = data/425;
	int tenk;
	int onek;
	int hund;
	int tens;
	int ones;
	
	tenk = data/10000;	//Ten Thousands Place
	data = data%10000;	//Remainder for Thousands Place
	onek = data/1000;	//Thousands Place
	data = data%1000;
	hund = data/100;	//Hundreds Place
	data = data%100;
	tens = data/10;		//Tens Place
	ones = data%10;		//Ones Place
	
	if (tenk != 0)
		LCD_putchar(tenk + '0');
	if (onek != 0)
		LCD_putchar(onek + '0');
	LCD_putchar(hund + '0');	
	LCD_putchar(0x2E);
	LCD_putchar(tens + '0');	
	LCD_putchar(ones + '0');
	return;
}
//Print unsigned long with decimal in correct place//
void LCD_current(int data)
{
	data = data*6902;
	data = data - 800000;
	unsigned short hunk;
	unsigned short tenk;
	unsigned short onek;
	unsigned short hund;
	unsigned short tens;
	unsigned short ones;
	hunk = data/100000;
	data = data%100000;
	tenk = data/10000;	//Ten Thousands Place
	data = data%10000;	//Remainder for Thousands Place
	onek = data/1000;	//Thousands Place
	data = data%1000;
	hund = data/100;	//Hundreds Place
	data = data%100;
	tens = data/10;		//Tens Place
	ones = data%10;		//Ones Place
	
	if (hunk != 0)
		LCD_putchar(hunk + '0');
	LCD_putchar(tenk + '0');
	LCD_putchar(onek + '0');
	LCD_putchar(0x2E);
	LCD_putchar(hund + '0');	
	LCD_putchar(tens + '0');	
	LCD_putchar(ones + '0');
	return;
}

//Print unsigned short as decimal number//
void LCD_dec(unsigned short data)
{
	unsigned short tenk;
	unsigned short onek;
	unsigned short hund;
	unsigned short tens;
	unsigned short ones;
	
	tenk = data/10000;	//Ten Thousands Place
	data = data%10000;	//Remainder for Thousands Place
	onek = data/1000;	//Thousands Place
	data = data%1000;
	hund = data/100;	//Hundreds Place
	data = data%100;
	tens = data/10;		//Tens Place
	ones = data%10;		//Ones Place
	
	if (tenk != 0)
		LCD_putchar(tenk + '0');
	if (tenk >= 1)
		LCD_putchar(onek + '0');
	if (hund != 0)
		LCD_putchar(hund + '0');	
	if (tens != 0)
		LCD_putchar(tens + '0');	
	LCD_putchar(ones + '0');	
	return;						
}

//Print char as a decimal number//
void LCD_dec(unsigned char data)
{
	unsigned char hund;
	unsigned char tens;
	unsigned char ones;
	
	hund = data/100;
	data = data%100;
	tens = data/10;
	ones = data%10;
	
	if (hund != 0)
		LCD_putchar(hund + '0');
	if (hund >= 1)
		LCD_putchar(tens + '0');
	if (hund == 0 && tens != 0)
		LCD_putchar(tens + '0');
	LCD_putchar(ones + '0');
	return;
}

//put the cursor in a specific position
///////////COLUMN 1//////COLUMN 20////////
//Line 1/// 0x00   ////// 0x13	  ////////
//LINE 2/// 0x40   ////// 0x53    ////////
//LINE 3/// 0x14   ////// 0x27	  ////////
//LINE 4/// 0x54   ////// 0x67    ////////
//////////////////////////////////////////
void LCD_cursor(char position)
{
	LCD_putchar(0xFE);
	LCD_putchar(0x45);
	LCD_putchar(position);
	delay_ms(10);
	return;
}

//Clear Display and Place cursor at line 1 column 1//
void LCD_clear(void)
{
	LCD_putchar(0xFE);
	LCD_putchar(0x51);
	delay_ms(10);
	return;
}
//Set Backlight Brightness:Brightness Value between 1 and 8//
void LCD_brightness(char brightness)
{
	if (brightness<=8 && brightness >=1)
	{
		LCD_putchar(0xFE);
		LCD_putchar(0x53);
		LCD_putchar(brightness);
		delay_ms(10);
	}
	else 
	{
		LCD_putchar(0xFE);
		LCD_putchar(0x53);
		LCD_putchar(0x04);
		delay_ms(10);
	}
}

